@@ -1,64 +1,61 @@
-''' This module contains the CommandLine, Option and Positional classes '''
+'''This module contains the CommandLine, Option and Positional classes.'''
 
-import re
 from collections import OrderedDict
+from types import NoneType
 
-from . import algo
 from .errors import CrazyError, CrazyTypeError
+from .str_utils import contains_space, is_valid_option_string, is_valid_variable_name
+
+# pylint: disable=redefined-builtin
+# pylint: disable=too-few-public-methods
+# pylint: disable=too-many-instance-attributes
+# pylint: disable=too-many-arguments
+# pylint: disable=too-many-positional-arguments
 
 class ExtendedBool:
+    '''Class that holds an extended bool.'''
+
     TRUE    = True
     FALSE   = False
     INHERIT = 'INHERIT'
 
-def _is_extended_bool(obj):
+def is_extended_bool(obj):
+    '''Check if `obj` an instance of `ExtendedBool`.'''
+
     return obj in (True, False, ExtendedBool.INHERIT)
 
-_VALID_OPTION_STRING_RE = re.compile('-[^\\s,]+')
-
-def _validate_option_string(option_string):
-    if not _VALID_OPTION_STRING_RE.fullmatch(option_string):
-        return False
-
-    if option_string == '--':
-        return False
-
-    return True
-
 class CommandLine:
-    '''
-    Represents a command line interface with options, positionals, and subcommands.
-    '''
+    '''Represents a command line interface with options, positionals, and subcommands.'''
 
     def __init__(self,
                  prog,
                  parent=None,
                  help=None,
-                 aliases=[],
+                 aliases=None,
                  abbreviate_commands=ExtendedBool.INHERIT,
                  abbreviate_options=ExtendedBool.INHERIT,
                  inherit_options=ExtendedBool.INHERIT):
-
-        '''
-        Initializes a CommandLine object with the specified parameters.
+        '''Initializes a CommandLine object with the specified parameters.
 
         Args:
             prog (str): The name of the program (or subcommand).
             help (str): The help message for the program (or subcommand).
             parent (CommandLine or None): The parent command line object, if any.
-            aliases (list of str): Aliases for this command.
+            aliases (list of str or None): Aliases for this command.
             abbreviate_commands (ExtendedBool): Specifies if commands can be abbreviated.
             abbreviate_options (ExtendedBool): Specifies if options can be abbreviated.
             inherit_options (ExtendedBool): Specifies if options are visible to subcommands.
         '''
+        if aliases is None:
+            aliases = []
 
         if not isinstance(prog, str):
             raise CrazyTypeError('prog', 'str', prog)
 
-        if not isinstance(parent, (CommandLine, None.__class__)):
+        if not isinstance(parent, (CommandLine, NoneType)):
             raise CrazyTypeError('parent', 'CommandLine|None', parent)
 
-        if not isinstance(help, (str, None.__class__)):
+        if not isinstance(help, (str, NoneType)):
             raise CrazyTypeError('help', 'str|None', help)
 
         if not isinstance(aliases, list):
@@ -68,13 +65,16 @@ class CommandLine:
             if not isinstance(alias, str):
                 raise CrazyTypeError(f'aliases[{index}]', 'str', alias)
 
-        if not _is_extended_bool(abbreviate_commands):
+            if contains_space(alias):
+                raise CrazyError(f'aliases[{index}]: cannot contain space')
+
+        if not is_extended_bool(abbreviate_commands):
             raise CrazyTypeError('abbreviate_commands', 'ExtendedBool', abbreviate_commands)
 
-        if not _is_extended_bool(abbreviate_options):
+        if not is_extended_bool(abbreviate_options):
             raise CrazyTypeError('abbreviate_options', 'ExtendedBool', abbreviate_options)
 
-        if not _is_extended_bool(inherit_options):
+        if not is_extended_bool(inherit_options):
             raise CrazyTypeError('inherit_options', 'ExtendedBool', inherit_options)
 
         self.prog = prog
@@ -88,96 +88,43 @@ class CommandLine:
         self.positionals = []
         self.subcommands = None
 
-    def add_option(self,
-            option_strings,
-            metavar=None,
-            help=None,
-            complete=None,
-            optional_arg=False,
-            groups=None,
-            repeatable=ExtendedBool.INHERIT,
-            final=False,
-            hidden=False,
-            when=None):
-        '''
-        Adds a new option to the command line.
+    def add_option(self, option_strings, **parameters):
+        '''Adds a new option to the command line.
 
-        Args:
-            option_strings (list of str): The list of option strings.
-            metavar (str): The metavar for the option.
-            help (str): The help message for the option.
-            complete (tuple): The completion specification for the option.
-            optional_arg (bool): Specifies if option's argument is optional.
-            groups (list of str): Specify to which mutually exclusive groups this option belongs to.
-            repeatable (ExtendedBool): Specifies if the option can be repeated.
-            when (str): Specifies a condition for showing this option.
+        For a list of valid parameters, see `class Option`.
 
         Returns:
             Option: The newly added Option object.
         '''
 
-        o = Option(self,
-                   option_strings,
-                   metavar=metavar,
-                   help=help,
-                   complete=complete,
-                   optional_arg=optional_arg,
-                   groups=groups,
-                   repeatable=repeatable,
-                   final=final,
-                   hidden=hidden,
-                   when=when)
+        o = Option(self, option_strings, **parameters)
         self.options.append(o)
         return o
 
-    def add_positional(self,
-            number,
-            metavar=None,
-            help=None,
-            repeatable=False,
-            complete=None,
-            when=None):
-        '''
-        Adds a new positional argument to the command line.
+    def add_positional(self, number, **parameters):
+        '''Adds a new positional argument to the command line.
 
-        Args:
-            number (int): The number of the positional argument (starting from 1)
-            metavar (str): The metavar for the positional.
-            help (str): The help message for the positional.
-            repeatable (bool): Specifies if positional can be specified more times
-            complete (tuple): The completion specification for the positional.
-            when (str): Specifies a condition for showing this positional.
+        For a list of valid parameters, see `class Positional`.
 
         Returns:
             Positional: The newly added Positional object.
         '''
 
-        p = Positional(self,
-                       number,
-                       metavar=metavar,
-                       help=help,
-                       repeatable=repeatable,
-                       complete=complete,
-                       when=when)
+        p = Positional(self, number, **parameters)
         self.positionals.append(p)
         return p
 
     def add_mutually_exclusive_group(self, group):
-        '''
-        Adds a new mutually exclusive group
+        '''Adds a new mutually exclusive group.
 
         Returns:
             MutuallyExclusiveGroup: The newly created mutually exclusive group.
         '''
+
         return MutuallyExclusiveGroup(self, group)
 
-    def add_subcommands(self, name='command', help=None):
-        '''
-        Adds a subcommands option to the command line if none exists already.
-
-        Args:
-            name (str): The name of the subcommands option.
-            help (str): The help message for the subcommands option.
+    def add_subcommands(self):
+        '''Adds a subcommands option to the command line if none exists already.
 
         Returns:
             SubCommandsOption: The newly created subcommands option.
@@ -185,33 +132,30 @@ class CommandLine:
         Raises:
             CrazyError: If the command line object already has subcommands.
         '''
-        if not isinstance(name, str):
-            raise CrazyTypeError('name', 'str', name)
-
-        if not isinstance(help, (str, None.__class__)):
-            raise CrazyTypeError('help', 'str|None', help)
 
         if self.subcommands:
             raise CrazyError('CommandLine object already has subcommands')
 
-        self.subcommands = SubCommandsOption(self, name, help)
+        self.subcommands = SubCommandsOption(self)
         return self.subcommands
 
     class OptionsGetter:
-        '''
-        A class for getting options from a CommandLine object.
-        '''
+        '''A class for getting options from a CommandLine object.'''
 
         def __init__(self, commandline, with_parent_options=False, only_with_arguments=False):
             self.options = OrderedDict()
 
-            commandlines = commandline.get_parents(include_self=True) if with_parent_options else [commandline]
-            for commandline in reversed(commandlines):
-                for option in commandline.options:
-                    if not only_with_arguments or option.complete:
-                        self.add(option, commandline)
+            if with_parent_options:
+                commandlines = commandline.get_parents(include_self=True)
+            else:
+                commandlines = [commandline]
 
-        def add(self, option, commandline):
+            for cmdline in reversed(commandlines):
+                for option in cmdline.options:
+                    if not only_with_arguments or option.complete:
+                        self._add(option, cmdline)
+
+        def _add(self, option, commandline):
             key = option.get_option_strings_key()
 
             if key not in self.options:
@@ -222,22 +166,25 @@ class CommandLine:
 
         def get(self):
             r = []
-            for commandline, options in self.options.values():
+            for _, options in self.options.values():
                 for option in options:
                     r.append(option)
             return r
 
     def get_options(self, with_parent_options=False, only_with_arguments=False):
-        '''
-        Gets a list of options associated with the command line.
+        '''Gets a list of options associated with the command line.
 
         Args:
-            with_parent_options (bool): If True, include options from parent command lines.
-            only_with_arguments (bool): If True, include only options that take arguments.
+            with_parent_options (bool):
+                If True, include options from parent command lines.
+
+            only_with_arguments (bool):
+                If True, include only options that take arguments.
 
         Returns:
             list: A list of Option objects
         '''
+
         assert isinstance(with_parent_options, bool), \
             "CommandLine.get_options: with_parent_options: expected bool, got %r" % with_parent_options
 
@@ -250,16 +197,19 @@ class CommandLine:
         return getter.get()
 
     def get_option_strings(self, with_parent_options=False, only_with_arguments=False):
-        '''
-        Gets a list of option strings associated with the command line.
+        '''Gets a list of option strings associated with the command line.
 
         Args:
-            with_parent_options (bool): If True, include options from parent command lines.
-            only_with_arguments (bool): If True, include only options that take arguments.
+            with_parent_options (bool):
+                If True, include options from parent command lines.
+
+            only_with_arguments (bool):
+                If True, include only options that take arguments.
 
         Returns:
             list: A list of option strings
         '''
+
         assert isinstance(with_parent_options, bool), \
             "CommandLine.get_option_strings: with_parent_options: expected bool, got %r" % with_parent_options
 
@@ -274,6 +224,8 @@ class CommandLine:
         return option_strings
 
     def get_final_options(self):
+        '''Gets a list of all final options.'''
+
         r = []
         for option in self.options:
             if option.final:
@@ -281,14 +233,15 @@ class CommandLine:
         return r
 
     def get_final_option_strings(self):
+        '''Gets a list of all final option strings.'''
+
         r = []
         for option in self.get_final_options():
             r.extend(option.option_strings)
         return r
 
     def get_positionals(self):
-        '''
-        Gets a list of positional arguments associated with the command line.
+        '''Gets a list of positional arguments associated with the command line.
 
         Note:
             SubCommandsOption objects are not considered positional arguments and are not included in the list.
@@ -296,20 +249,20 @@ class CommandLine:
         Returns:
             list: A list of positional arguments
         '''
+
         return list(self.positionals)
 
-    def get_subcommands_option(self):
-        '''
-        Gets the subcommands option of the command line.
+    def get_subcommands(self):
+        '''Gets the subcommands of the command line.
 
         Returns:
             SubCommandsOption or None: The subcommands option if it exists, otherwise None.
         '''
+
         return self.subcommands
 
     def get_parents(self, include_self=False):
-        '''
-        Gets a list of parent CommandLine objects.
+        '''Gets a list of parent CommandLine objects.
 
         Args:
             include_self (bool): If True, includes the current CommandLine object in the list.
@@ -317,6 +270,7 @@ class CommandLine:
         Returns:
             list: A list of parent CommandLine objects.
         '''
+
         assert isinstance(include_self, bool), \
             "CommandLine.get_parents: include_self: expected bool, got %r" % include_self
 
@@ -333,6 +287,8 @@ class CommandLine:
         return parents
 
     def get_options_by_option_strings(self, option_strings):
+        '''Return all options containing one option_strings.'''
+
         result = []
 
         for option_string in option_strings:
@@ -349,6 +305,8 @@ class CommandLine:
         return result
 
     def get_highest_positional_num(self):
+        '''Get the highest positional number.'''
+
         highest = 0
         for positional in self.positionals:
             highest = max(highest, positional.number)
@@ -357,29 +315,38 @@ class CommandLine:
         return highest
 
     def get_program_name(self):
+        '''Return the program name.'''
+
         commandlines = self.get_parents(include_self=True)
         return commandlines[0].prog
 
-    def get_command_path(self):
-        cmd = ' '.join(c.prog for c in self.get_parents(include_self=True))
-        return cmd
+    def get_command_path(self, progname=None):
+        '''Return the full command path.'''
+        prognames = [c.prog for c in self.get_parents(include_self=True)]
 
-    def get_all_commands(self, with_aliases=True):
-        r = [self.prog]
-        if with_aliases:
-            r.extend(self.aliases)
-        return r
+        if progname is not None:
+            prognames[0] = progname
+
+        return ' '.join(prognames)
 
     def visit_commandlines(self, callback):
+        '''Apply a callback to all CommandLine objects.'''
+
         callback(self)
-        if self.get_subcommands_option():
-            for sub in self.get_subcommands_option().subcommands:
+        if self.get_subcommands():
+            for sub in self.get_subcommands().subcommands:
                 sub.visit_commandlines(callback)
 
+    def get_all_commandlines(self):
+        '''Get a list of all defined commandlines.'''
+
+        result = []
+        self.visit_commandlines(result.append)
+        return result
+
     def copy(self):
-        '''
-        Make a copy of the current CommandLine object, including sub-objects.
-        '''
+        '''Make a copy of the current CommandLine object, including sub-objects.'''
+
         copy = CommandLine(
             self.prog,
             parent              = None,
@@ -400,7 +367,8 @@ class CommandLine:
                 repeatable      = option.repeatable,
                 final           = option.final,
                 hidden          = option.hidden,
-                when            = option.when
+                when            = option.when,
+                capture         = option.capture
             )
 
         for positional in self.positionals:
@@ -410,11 +378,12 @@ class CommandLine:
                 help            = positional.help,
                 repeatable      = positional.repeatable,
                 complete        = positional.complete,
-                when            = positional.when
+                when            = positional.when,
+                capture         = positional.capture
             )
 
         if self.subcommands is not None:
-            subcommands_option = copy.add_subcommands(self.subcommands.metavar, self.subcommands.help)
+            subcommands_option = copy.add_subcommands()
             for subparser in self.subcommands.subcommands:
                 subcommands_option.add_commandline_object(subparser.copy())
 
@@ -422,15 +391,15 @@ class CommandLine:
 
     def __eq__(self, other):
         return (
-            isinstance(other, CommandLine) and
-            self.prog                == other.prog and
-            self.aliases             == other.aliases and
-            self.help                == other.help and
+            isinstance(other, CommandLine)                        and
+            self.prog                == other.prog                and
+            self.aliases             == other.aliases             and
+            self.help                == other.help                and
             self.abbreviate_commands == other.abbreviate_commands and
-            self.abbreviate_options  == other.abbreviate_options and
-            self.inherit_options     == other.inherit_options and
-            self.options             == other.options and
-            self.positionals         == other.positionals and
+            self.abbreviate_options  == other.abbreviate_options  and
+            self.inherit_options     == other.inherit_options     and
+            self.options             == other.options             and
+            self.positionals         == other.positionals         and
             self.subcommands         == other.subcommands
         )
 
@@ -439,6 +408,8 @@ class CommandLine:
             self.prog, self.help, self.abbreviate_commands, self.options, self.positionals, self.subcommands)
 
 class Positional:
+    '''Class representing a command line positional.'''
+
     def __init__(
             self,
             parent,
@@ -447,29 +418,48 @@ class Positional:
             help=None,
             complete=None,
             repeatable=False,
-            when=None):
+            when=None,
+            capture=None):
+        '''Initializes a Positional object with the specified parameters.
 
-        if not isinstance(parent, (CommandLine, None.__class__)):
+        Args:
+            parent (CommandLine): The parent command line object, if any.
+            number (int): The number of the positional argument (starting from 1)
+            metavar (str): The metavar for the positional.
+            help (str): The help message for the positional.
+            repeatable (bool): Specifies if positional can be specified more times
+            complete (list): The completion specification for the positional.
+            when (str): Specifies a condition for showing this positional.
+            capture (str): Specifies the variable name for capturing
+        '''
+
+        if not isinstance(parent, (CommandLine, NoneType)):
             raise CrazyTypeError('parent', 'CommandLine|None', parent)
 
         if not isinstance(number, int):
             raise CrazyTypeError('number', 'int', number)
 
-        if not isinstance(metavar, (str, None.__class__)):
+        if not isinstance(metavar, (str, NoneType)):
             raise CrazyTypeError('metavar', 'str|None', metavar)
 
-        if not isinstance(help, (str, None.__class__)):
+        if not isinstance(help, (str, NoneType)):
             raise CrazyTypeError('help', 'str|None', help)
 
-        if not isinstance(complete, (list, tuple, None.__class__)):
+        if not isinstance(complete, (list, tuple, NoneType)):
             raise CrazyTypeError('complete', 'list|None', complete)
 
         if not isinstance(repeatable, bool):
             raise CrazyTypeError('repeatable', 'bool', repeatable)
 
-        if not isinstance(when, (str, None.__class__)):
+        if not isinstance(when, (str, NoneType)):
             raise CrazyTypeError('when', 'str|None', when)
 
+        if not isinstance(capture, (str, NoneType)):
+            raise CrazyTypeError('capture', 'str|None', capture)
+
+        if capture is not None and not is_valid_variable_name(capture):
+            raise CrazyError(f"Invalid variable name: {capture!r}")
+
         if number <= 0:
             raise CrazyError(f'number: value ({number}) is invalid, number has to be >= 1')
 
@@ -480,15 +470,16 @@ class Positional:
         self.repeatable = repeatable
         self.complete = complete if complete else ['none']
         self.when = when
+        self.capture = capture
 
     def get_positional_index(self):
-        '''
-        Returns the index of the current positional argument within the current
+        '''Returns the index of the current positional argument within the current
         commandline, including parent commandlines.
 
         Returns:
             int: The index of the positional argument.
         '''
+
         positional_no = self.number - 1
 
         for commandline in self.parent.get_parents():
@@ -497,14 +488,13 @@ class Positional:
                 highest = max(highest, positional.number)
             positional_no += highest
 
-            if commandline.get_subcommands_option():
+            if commandline.get_subcommands():
                 positional_no += 1
 
         return positional_no
 
     def get_positional_num(self):
-        '''
-        Returns the number of the current positional argument within the current
+        '''Returns the number of the current positional argument within the current
         commandline, including parent commandlines.
 
         Note:
@@ -513,20 +503,24 @@ class Positional:
         Returns:
             int: The number of the positional argument.
         '''
+
         return self.get_positional_index() + 1
 
     def __eq__(self, other):
         return (
-            isinstance(other, Positional) and
-            self.number          == other.number and
-            self.metavar         == other.metavar and
-            self.help            == other.help and
-            self.repeatable      == other.repeatable and
-            self.complete        == other.complete and
-            self.when            == other.when
+            isinstance(other, Positional)       and
+            self.number     == other.number     and
+            self.metavar    == other.metavar    and
+            self.help       == other.help       and
+            self.repeatable == other.repeatable and
+            self.complete   == other.complete   and
+            self.when       == other.when       and
+            self.capture    == other.capture
         )
 
 class Option:
+    '''Class representing a command line option.'''
+
     def __init__(
             self,
             parent,
@@ -539,23 +533,44 @@ class Option:
             repeatable=ExtendedBool.INHERIT,
             final=False,
             hidden=False,
-            when=None):
-        if not isinstance(parent, (CommandLine, None.__class__)):
+            when=None,
+            capture=None):
+        '''Initializes an Option object with the specified parameters.
+
+        Args:
+            parent (CommandLine): The parent command line object, if any.
+            option_strings (list of str): The list of option strings.
+            metavar (str): The metavar for the option.
+            help (str): The help message for the option.
+            complete (tuple): The completion specification for the option.
+            optional_arg (bool): Specifies if option's argument is optional.
+            groups (list of str): Specify to which mutually exclusive groups this option belongs to.
+            repeatable (ExtendedBool): Specifies if the option can be repeated.
+            final (bool): If True, no more options are suggested after this one.
+            hidden (bool): Specifies if this option is hidden.
+            when (str): Specifies a condition for showing this option.
+            capture (str): Specifies the variable name for capturing
+
+        Returns:
+            Option: The newly added Option object.
+        '''
+
+        if not isinstance(parent, (CommandLine, NoneType)):
             raise CrazyTypeError('parent', 'CommandLine|None', parent)
 
         if not isinstance(option_strings, list):
             raise CrazyTypeError('option_strings', 'list', option_strings)
 
-        if not isinstance(metavar, (str, None.__class__)):
+        if not isinstance(metavar, (str, NoneType)):
             raise CrazyTypeError('metavar', 'str|None', metavar)
 
-        if not isinstance(help, (str, None.__class__)):
+        if not isinstance(help, (str, NoneType)):
             raise CrazyTypeError('help', 'str|None', help)
 
-        if not isinstance(complete, (list, tuple, None.__class__)):
+        if not isinstance(complete, (list, tuple, NoneType)):
             raise CrazyTypeError('complete', 'list|None', complete)
 
-        if not isinstance(groups, (list, None.__class__)):
+        if not isinstance(groups, (list, NoneType)):
             raise CrazyTypeError('groups', 'list|None', groups)
 
         if groups is not None:
@@ -566,7 +581,7 @@ class Option:
         if not isinstance(optional_arg, bool):
             raise CrazyTypeError('optional_arg', 'bool', optional_arg)
 
-        if not _is_extended_bool(repeatable):
+        if not is_extended_bool(repeatable):
             raise CrazyTypeError('repeatable', 'ExtendedBool', repeatable)
 
         if not isinstance(final, bool):
@@ -575,15 +590,24 @@ class Option:
         if not isinstance(hidden, bool):
             raise CrazyTypeError('hidden', 'bool', hidden)
 
-        if not isinstance(when, (str, None.__class__)):
+        if not isinstance(when, (str, NoneType)):
             raise CrazyTypeError('when', 'str|None', when)
 
+        if not isinstance(capture, (str, NoneType)):
+            raise CrazyTypeError('capture', 'str|None', capture)
+
         if not option_strings:
             raise CrazyError('Empty option strings')
 
-        for option_string in option_strings:
-            if not _validate_option_string(option_string):
-                raise CrazyError(f"Invalid option string: {option_string}")
+        for index, option_string in enumerate(option_strings):
+            if not isinstance(option_string, str):
+                raise CrazyTypeError(f'option_strings[{index}]', 'str', option_string)
+
+            if not is_valid_option_string(option_string):
+                raise CrazyError(f"Invalid option string: {option_string!r}")
+
+        if capture is not None and not is_valid_variable_name(capture):
+            raise CrazyError(f"Invalid variable name: {capture!r}")
 
         if metavar and not complete:
             raise CrazyError(f'Option {option_strings} has metavar set, but has no complete')
@@ -602,53 +626,56 @@ class Option:
         self.final = final
         self.hidden = hidden
         self.when = when
+        self.capture = capture
 
     def get_option_strings(self):
-        '''
-        Returns the option strings associated with the Option object.
+        '''Returns the option strings associated with the Option object.
 
         Returns:
             list: A list of strings representing the option strings.
         '''
+
         return self.option_strings
 
     def get_option_strings_key(self, delimiter=' '):
+        '''Returns a key with consisting of all option strings.'''
+
         return delimiter.join(sorted(self.option_strings))
 
     def get_short_option_strings(self):
-        '''
-        Returns the short option strings associated with the Option object.
+        '''Returns the short option strings associated with the Option object.
 
         Returns:
             list: A list of short option strings ("-o").
         '''
+
         return [o for o in self.option_strings if o.startswith('-') and len(o) == 2]
 
     def get_long_option_strings(self):
-        '''
-        Returns the long option strings associated with the Option object.
+        '''Returns the long option strings associated with the Option object.
 
         Returns:
             list: A list of long option strings ("--option").
         '''
+
         return [o for o in self.option_strings if o.startswith('--')]
 
     def get_old_option_strings(self):
-        '''
-        Returns the old-style option strings associated with the Option object.
+        '''Returns the old-style option strings associated with the Option object.
 
         Returns:
             list: A list of old-style option strings ("-option").
         '''
+
         return [o for o in self.option_strings if o.startswith('-') and not o.startswith('--') and len(o) > 2]
 
     def get_conflicting_options(self):
-        '''
-        Returns a list of conflicting options within the same mutually exclusive groups.
+        '''Returns a list of conflicting options within the same mutually exclusive groups.
 
         Returns:
             list: A list of Option objects representing conflicting options.
         '''
+
         if not self.groups:
             return []
         r = []
@@ -661,31 +688,32 @@ class Option:
         return r
 
     def get_conflicting_option_strings(self):
-        '''
-        Returns a list of option strings conflicting with the current option
+        '''Returns a list of option strings conflicting with the current option
         within the same mutually exclusive groups.
 
         Returns:
             list: A list of option strings representing conflicting options.
         '''
-        option_strings = []
+
+        r = []
         for option in self.get_conflicting_options():
-            option_strings.extend(option.get_option_strings())
-        return option_strings
+            r.extend(option.get_option_strings())
+        return r
 
     def __eq__(self, other):
         return (
-            isinstance(other, Option) and
-            self.option_strings  == other.option_strings and
-            self.metavar         == other.metavar and
-            self.help            == other.help and
-            self.optional_arg    == other.optional_arg and
-            self.repeatable      == other.repeatable and
-            self.final           == other.final and
-            self.hidden          == other.hidden and
-            self.complete        == other.complete and
-            self.groups          == other.groups and
-            self.when            == other.when
+            isinstance(other, Option)                   and
+            self.option_strings == other.option_strings and
+            self.metavar        == other.metavar        and
+            self.help           == other.help           and
+            self.optional_arg   == other.optional_arg   and
+            self.repeatable     == other.repeatable     and
+            self.final          == other.final          and
+            self.hidden         == other.hidden         and
+            self.complete       == other.complete       and
+            self.groups         == other.groups         and
+            self.when           == other.when           and
+            self.capture        == other.capture
         )
 
     def __repr__(self):
@@ -693,25 +721,38 @@ class Option:
             self.option_strings, self.metavar, self.help)
 
 class SubCommandsOption(Positional):
-    def __init__(self, parent, name, help):
+    '''Class holding subcommands of a commandline.'''
+
+    def __init__(self, parent):
         self.subcommands = []
 
         super().__init__(
             parent,
             parent.get_highest_positional_num() + 1,
-            metavar='command',
-            help=help)
+            metavar='command')
 
     def add_commandline_object(self, commandline):
+        '''Add an existing commandline object as a subcommand.'''
+
+        if self.get_subcommand_by_name(commandline.prog):
+            raise CrazyError(f'Duplicate subcommand. {commandline.prog} already defined.')
+
         commandline.parent = self.parent
         self.subcommands.append(commandline)
 
     def add_commandline(self, name, help=''):
+        '''Create a new commandline object and add it as a subcommand.'''
+
+        if self.get_subcommand_by_name(name):
+            raise CrazyError(f'Duplicate subcommand. {name} already defined.')
+
         commandline = CommandLine(name, help=help, parent=self.parent)
         self.subcommands.append(commandline)
         return commandline
 
     def get_choices(self, with_aliases=True):
+        '''Return a mapping of command and its description.'''
+
         r = OrderedDict()
         for subcommand in self.subcommands:
             r[subcommand.prog] = subcommand.help
@@ -720,20 +761,29 @@ class SubCommandsOption(Positional):
                     r[alias] = subcommand.help
         return r
 
+    def get_subcommand_by_name(self, name):
+        '''Return a subcommand by name or None if not found.'''
+
+        for subcommand in self.subcommands:
+            if subcommand.prog == name:
+                return subcommand
+        return None
+
     def __eq__(self, other):
         return (
-            isinstance(other, SubCommandsOption) and
-            self.subcommands    == other.subcommands and
-            self.help           == other.help and
-            self.metavar        == other.metavar and
-            self.complete       == other.complete
+            isinstance(other, SubCommandsOption)  and
+            self.subcommands == other.subcommands and
+            self.help        == other.help        and
+            self.metavar     == other.metavar     and
+            self.complete    == other.complete
         )
 
     def __repr__(self):
-        return '{help: %r, subcommands %r}' % (
-            self.help, self.subcommands)
+        return '{help: %r, subcommands %r}' % (self.help, self.subcommands)
 
 class MutuallyExclusiveGroup:
+    '''Helper class for adding mutually exclusive options.'''
+
     def __init__(self, parent, group):
         assert isinstance(parent, CommandLine), \
             "MutuallyExclusiveGroup: parent: expected CommandLine, got %r" % parent
@@ -744,30 +794,25 @@ class MutuallyExclusiveGroup:
         self.parent = parent
         self.group = group
 
-    def add(self,
-            option_strings,
-            metavar=None,
-            help=None,
-            complete=None,
-            optional_arg=False,
-            repeatable=ExtendedBool.INHERIT,
-            final=False,
-            hidden=False,
-            when=None):
-        ''' Creates and adds a new option '''
+    def add(self, option_strings, **parameters):
+        '''Creates and adds a new option.
+
+        For a list of valid parameters, see `class Option`.
+        '''
+
+        if 'groups' in parameters:
+            raise CrazyError('Paramter `groups` not allowed')
+
+        if 'group' in parameters:
+            raise CrazyError('Paramter `group` not allowed')
+
         return self.parent.add_option(
             option_strings,
-            metavar=metavar,
-            help=help,
-            complete=complete,
-            optional_arg=optional_arg,
             groups=[self.group],
-            repeatable=repeatable,
-            final=final,
-            hidden=hidden,
-            when=when)
+            **parameters)
 
     def add_option(self, option):
-        ''' Adds an option object '''
+        '''Adds an option object.'''
+
         option.parent = self.parent
         option.groups = [self.group]
